###############
# main.pm
#
# Copyright 2007 Francisco Amato
#
# This file is part of isr-evilgrade, www.infobyte.com.ar .
#
# isr-evilgrade is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
#
# isr-evilgrade is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with w3af; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# '''
##
package isrcore::main;

use strict;

#external modules
use Data::Dump qw(dump);

my $options = {
    'port'  => { 'val' => 80, 'desc' => 'Webserver listening port'},
    'debug'  => { 'val' => 0, 'desc' => 'Debug mode'},    
};
my $base = {
    'path' => 'modules',
    'pwd'	 => './',
    'options' => $options
};
    
##########################################################################
# FUNCTION	new
# RECEIVES
# RETURNS
# EXPECTS
# DOES		class's constructor
sub new {
    my $class = shift;
    my $self = {'Base' => $base, @_ };

    return bless $self, $class;
}

##########################################################################
# FUNCTION	loadmodules
# RECEIVES
# RETURNS
# EXPECTS
# DOES		module's loader
sub loadmodules{

    my $self = shift;
    my @modules;

    my $path = $self->{'Base'}->{'pwd'}.$self->{'Base'}->{'path'};

    local *DIR;
    if (!opendir(DIR,"$path")){
	return "[LOADMODULES] - (*) No such file or directory ($path)";
    }
    my @files = grep(!/^\.\.?$/,readdir(DIR));
    closedir(DIR);
    
    my $modules;
    my $mods=0;
    foreach my $f (@files)
    {
	my $base = $self->{'Base'}->{'path'}."/".$f;
	my ($name) = $f =~ m/([^\.]+)\.pm/;

	delete $INC{$self->{'Base'}->{'path'}."/".$name};

	$self->printd("Loading module: $base\n");
    	my $result = do($base);
			
	if ($@) {
	    $self->println('Error: Loading module ($base):' . $@);
	    delete $INC{$self->{'Base'}->{'path'}."/".$name};
	    next;
	}
	if (!$result){
	    $self->println("Error: module ($base) did not return true\n");
	    next;
	}
	my $object = "$self->{'Base'}->{'path'}::$name";
	my $module;
	if (eval { $module = $object->new()}){ #Verify object's creation
	    $mods++;
	    $module = $object->new();
	    $modules->{$name} = $module;
	}else{
	    $self->println("Error: module ($base): $@\n");
	}

    }
    $self->println("--------------------------------------\n");    
    $self->println("-- |  [ISR]  | www.infobyte.com.ar |\n");
    $self->println("--------------------------------------\n\n");
    $self->println("- $mods modules loaded.\n");
    
    delete $self->{'modules'};
    $self->{'modules'} = $modules;
    return 1;
}

##########################################################################
# FUNCTION	println
# RECEIVES	msg
# RETURNS	
# EXPECTS
# DOES		print console msg
sub println{
    my $self = shift;
    my ($msg) = @_;
    printf $msg;
}

##########################################################################
# FUNCTION	printd
# RECEIVES	msg
# RETURNS
# EXPECTS
# DOES		print console debug msg
sub printd{
    my $self = shift;
    my ($msg) = @_;
#    print dump($self);
    printf "[DEBUG] - " . $msg if ($self->{'Base'}->{'options'}->{'debug'}->{'val'});
}

##########################################################################
# FUNCTION	ptitle
# RECEIVES	msg
# RETURNS
# EXPECTS
# DOES		print console debug msg
sub ptitle{
    my $self = shift;
    my ($msg) = @_;
    printf "\n$msg:\n";
    printf "="x length($msg);
    printf "\n\n";
}

1;
